"use strict";
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const xml2js = __importStar(require("xml2js"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const fileutils_1 = __importDefault(require("../../utils/fileutils"));
if (Symbol["asyncIterator"] === undefined) {
    // tslint:disable-next-line:no-any
    Symbol["asyncIterator"] = Symbol.for("asyncIterator");
}
const STANDARD_VALUE_SETS = [
    "AccountContactMultiRoles",
    "AccountContactRole",
    "AccountOwnership",
    "AccountRating",
    "AccountType",
    "AddressCountryCode",
    "AddressStateCode",
    "AssetStatus",
    "CampaignMemberStatus",
    "CampaignStatus",
    "CampaignType",
    "CaseContactRole",
    "CaseOrigin",
    "CasePriority",
    "CaseReason",
    "CaseStatus",
    "CaseType",
    "ContactRole",
    "ContractContactRole",
    "ContractStatus",
    "EntitlementType",
    "EventSubject",
    "EventType",
    "FiscalYearPeriodName",
    "FiscalYearPeriodPrefix",
    "FiscalYearQuarterName",
    "FiscalYearQuarterPrefix",
    "IdeaCategory",
    "IdeaMultiCategory",
    "IdeaStatus",
    "IdeaThemeStatus",
    "Industry",
    "InvoiceStatus",
    "LeadSource",
    "LeadStatus",
    "OpportunityCompetitor",
    "OpportunityStage",
    "OpportunityType",
    "OrderStatus",
    "OrderType",
    "PartnerRole",
    "Product2Family",
    "QuestionOrigin",
    "QuickTextCategory",
    "QuickTextChannel",
    "QuoteStatus",
    "SalesTeamRole",
    "Salutation",
    "ServiceContractApprovalStatus",
    "SocialPostClassification",
    "SocialPostEngagementLevel",
    "SocialPostReviewedStatus",
    "SolutionStatus",
    "TaskPriority",
    "TaskStatus",
    "TaskSubject",
    "TaskType",
    "WorkOrderLineItemStatus",
    "WorkOrderPriority",
    "WorkOrderStatus"
];
/**
 * This code was adapted from github:sfdx-jayree-plugin project which was
 * based on the original github:sfdx-hydrate project
 */
class Packagexml {
    constructor(conn, configs) {
        this.packageTypes = {};
        this.conn = conn;
        this.configs = configs;
        this.result = [];
    }
    async build() {
        try {
            const folders = [];
            const unfolderedObjects = [];
            await this.describeMetadata(unfolderedObjects, folders);
            await this.buildInstalledPackageRegex();
            await this.handleUnfolderedObjects(unfolderedObjects);
            await this.handleFolderedObjects(folders);
            if (!this.packageTypes["StandardValueSet"]) {
                this.packageTypes["StandardValueSet"] = [];
            }
            STANDARD_VALUE_SETS.forEach(member => {
                this.packageTypes["StandardValueSet"].push(member);
                this.result.push({
                    type: "StandardValueSet",
                    fullName: member
                });
            });
            let packageXml = this.generateXml();
            let dir = path.parse(this.configs.outputFile).dir;
            if (!fs.existsSync(dir)) {
                fileutils_1.default.mkDirByPathSync(dir);
            }
            fs.writeFileSync(this.configs.outputFile, packageXml);
            return packageXml;
        }
        catch (err) {
            console.log(err);
        }
    }
    async buildInstalledPackageRegex() {
        // fetch and execute installed package promise to build regex
        let ipRegexStr = "^(";
        if (this.ipPromise) {
            this.ipPromise.then(instPack => {
                instPack.forEach(pkg => {
                    ipRegexStr += pkg.namespacePrefix + "|";
                });
                ipRegexStr += ")+__";
                this.ipRegex = RegExp(ipRegexStr);
            });
        }
        else {
            this.ipRegex = RegExp("");
        }
    }
    async describeMetadata(unfolderedObjects, folders) {
        var e_1, _a;
        const describe = await this.conn.metadata.describe(this.configs.apiVersion);
        try {
            for (var _b = __asyncValues(describe.metadataObjects), _c; _c = await _b.next(), !_c.done;) {
                const object = _c.value;
                if (this.configs.quickFilters.length !== 0 &&
                    !this.configs.quickFilters.includes(object.xmlName)) {
                    continue;
                }
                if (object.inFolder) {
                    const objectType = object.xmlName.replace("Template", "");
                    const promise = this.conn.metadata.list({
                        type: `${objectType}Folder`
                    }, this.configs.apiVersion);
                    folders.push(promise);
                }
                else {
                    const promise = this.conn.metadata.list({
                        type: object.xmlName
                    }, this.configs.apiVersion);
                    if (object.xmlName === "InstalledPackage") {
                        this.ipPromise = promise.then(); // clone promise
                    }
                    unfolderedObjects.push(promise);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    generateXml() {
        const packageJson = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            types: [],
            version: this.configs.apiVersion
        };
        Object.keys(this.packageTypes).forEach(mdtype => {
            if (this.configs.quickFilters.length === 0 ||
                this.configs.quickFilters.includes(mdtype)) {
                packageJson.types.push({
                    name: mdtype,
                    members: this.packageTypes[mdtype].sort()
                });
            }
        });
        const builder = new xml2js.Builder({
            xmldec: { version: "1.0", encoding: "utf-8" }
        });
        let packageObj = {
            Package: packageJson
        };
        let packageXml = builder.buildObject(packageObj);
        return packageXml;
    }
    async handleFolderedObjects(folders) {
        var e_2, _a, e_3, _b;
        const folderedObjects = [];
        try {
            for (var folders_1 = __asyncValues(folders), folders_1_1; folders_1_1 = await folders_1.next(), !folders_1_1.done;) {
                const folder = folders_1_1.value;
                let folderItems = [];
                if (Array.isArray(folder)) {
                    folderItems = folder;
                }
                else if (folder) {
                    folderItems = [folder];
                }
                if (folderItems.length > 0) {
                    try {
                        for (var folderItems_1 = __asyncValues(folderItems), folderItems_1_1; folderItems_1_1 = await folderItems_1.next(), !folderItems_1_1.done;) {
                            const folderItem = folderItems_1_1.value;
                            if (folderItem) {
                                this.result.push(folderItem);
                                let objectType = folderItem.type.replace("Folder", "");
                                if (objectType === "Email") {
                                    objectType += "Template";
                                }
                                this.addMember(objectType, folderItem);
                                const promise = this.conn.metadata.list({
                                    type: objectType,
                                    folder: folderItem.fullName
                                }, this.configs.apiVersion);
                                folderedObjects.push(promise);
                            }
                        }
                    }
                    catch (e_3_1) { e_3 = { error: e_3_1 }; }
                    finally {
                        try {
                            if (folderItems_1_1 && !folderItems_1_1.done && (_b = folderItems_1.return)) await _b.call(folderItems_1);
                        }
                        finally { if (e_3) throw e_3.error; }
                    }
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (folders_1_1 && !folders_1_1.done && (_a = folders_1.return)) await _a.call(folders_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        (await Promise.all(folderedObjects)).forEach(folderedObject => {
            try {
                if (folderedObject) {
                    let folderedObjectItems = [];
                    if (Array.isArray(folderedObject)) {
                        folderedObjectItems = folderedObject;
                    }
                    else {
                        folderedObjectItems = [folderedObject];
                    }
                    folderedObjectItems.forEach(metadataEntries => {
                        if (metadataEntries) {
                            this.addMember(metadataEntries.type, metadataEntries);
                            this.result.push(metadataEntries);
                        }
                        else {
                            console.log("No metadataEntry available");
                        }
                    });
                }
            }
            catch (err) {
                console.log(err);
            }
        });
    }
    async handleUnfolderedObjects(unfolderedObjects) {
        (await Promise.all(unfolderedObjects)).forEach(unfolderedObject => {
            try {
                if (unfolderedObject) {
                    let unfolderedObjectItems = [];
                    if (Array.isArray(unfolderedObject)) {
                        unfolderedObjectItems = unfolderedObject;
                    }
                    else {
                        unfolderedObjectItems = [unfolderedObject];
                    }
                    unfolderedObjectItems.forEach(metadataEntries => {
                        if (metadataEntries) {
                            this.addMember(metadataEntries.type, metadataEntries);
                            this.result.push(metadataEntries);
                        }
                        else {
                            console.log("No metadataEntry available");
                        }
                    });
                }
            }
            catch (err) {
                console.log(err);
            }
        });
    }
    addMember(type, member) {
        /**
         * Managed package - fullName starts with 'namespacePrefix__' || namespacePrefix is not null || manageableState = installed
         * Unmanaged package - manageableState = unmanaged
         * Regular custom objects - manageableState = unmanaged or undefined
         */
        if (type &&
            !(typeof type === "object") &&
            !(this.configs.excludeManaged &&
                (this.ipRegex.test(member.fullName) ||
                    member.namespacePrefix ||
                    member.manageableState === "installed"))) {
            try {
                if (member.fileName.includes("ValueSetTranslation")) {
                    const x = member.fileName
                        .split(".")[1]
                        .substring(0, 1)
                        .toUpperCase() + member.fileName.split(".")[1].substring(1);
                    if (!this.packageTypes[x]) {
                        this.packageTypes[x] = [];
                    }
                    this.packageTypes[x].push(member.fullName);
                }
                else {
                    if (!this.packageTypes[type]) {
                        this.packageTypes[type] = [];
                    }
                    this.packageTypes[type].push(member.fullName);
                }
            }
            catch (ex) {
                console.log("Type " + JSON.stringify(type));
            }
        }
    }
}
exports.Packagexml = Packagexml;
class BuildConfig {
    constructor(flags, apiVersion) {
        // flags always take precendence over configs from file
        this.excludeManaged = flags["excludemanaged"];
        this.apiVersion = flags["apiversion"] || apiVersion;
        this.quickFilters = flags["quickfilter"]
            ? flags["quickfilter"].split(",").map(elem => {
                return elem.trim();
            })
            : [];
        this.outputFile = flags["outputfile"] || "package.xml";
    }
}
exports.BuildConfig = BuildConfig;
//# sourceMappingURL=packageBuilder.js.map